@@ -2,12 +2,11 @@
 
 import datetime
 import os
-import sys
-
-import pkg_resources
 
 from sphinx.ext import autodoc
 
+import vdirsyncer
+
 extensions = ['sphinx.ext.autodoc']
 
 templates_path = ['_templates']
@@ -19,15 +18,7 @@ project = u'vdirsyncer'
 copyright = (u'2014-{}, Markus Unterwaditzer & contributors'
              .format(datetime.date.today().strftime('%Y')))
 
-try:
-    # The full version, including alpha/beta/rc tags.
-    release = pkg_resources.require('vdirsyncer')[0].version
-except pkg_resources.DistributionNotFound:
-    print('To build the documentation, the distribution information of '
-          'vdirsyncer has to be available. Run "setup.py develop" to do '
-          'this.')
-    sys.exit(1)
-
+release = vdirsyncer.__version__
 version = '.'.join(release.split('.')[:2])  # The short X.Y version.
 
 exclude_patterns = ['_build']
@@ -101,11 +92,24 @@ class StorageDocumenter(autodoc.ClassDocumenter):
     Sphinx' __init__ signature removed.'''
 
     objtype = 'storage'
-    directivetype = 'attribute'
+    domain = None
+    directivetype = 'storage'
+    option_spec = {}
+
+    @classmethod
+    def can_document_member(cls, member, membername, isattr, parent):
+        from vdirsyncer.storage.base import Storage
+        return isinstance(member, Storage)
 
     def format_signature(self):
         return ''
 
+    def add_directive_header(self, sig):
+        directive = getattr(self, 'directivetype', self.objtype)
+        name = self.object.storage_name
+        self.add_line(u'.. %s:: %s%s' % (directive, name, sig),
+                      '<autodoc>')
+
     def get_doc(self, encoding=None, ignore=1):
         from vdirsyncer.cli.utils import format_storage_config
         rv = autodoc.ClassDocumenter.get_doc(self, encoding, ignore)
@@ -115,6 +119,9 @@ class StorageDocumenter(autodoc.ClassDocumenter):
 
 
 def setup(app):
+    from sphinx.domains.python import PyObject
+    app.add_object_type('storage', 'storage', 'pair: %s; storage',
+                        doc_field_types=PyObject.doc_field_types)
     app.add_role('gh', github_issue_role)
     app.add_role('ghpr', github_issue_role)
     app.add_autodocumenter(StorageDocumenter)
