@@ -32,28 +32,22 @@ cfg['install'] = [script("""
 """)]
 
 cfg['script'] = [script("""
-    make -e $BUILD
+    make -e $BUILD;
 """)]
 
 matrix = []
 cfg['matrix'] = {'include': matrix}
 
-for python in ("2.7", "3.3", "3.4", "3.5", "pypy"):
-    matrix.append({
-        'python': python,
-        'env': 'BUILD=style BUILD_PRS=true'
-    })
+matrix.append({
+    'python': "3.5",
+    'env': 'BUILD=style BUILD_PRS=true'
+})
 
+for python in ("3.3", "3.4", "3.5"):
     if python == "3.5":
         dav_servers = ("radicale", "owncloud", "nextcloud", "baikal",
                        "davical")
         rs_servers = ("mysteryshack",)
-    elif python == "2.7":
-        dav_servers = ("owncloud", "nextcloud", "baikal", "davical")
-        rs_servers = ("mysteryshack",)
-    elif python == "pypy":
-        dav_servers = ()
-        rs_servers = ()
     else:
         dav_servers = ("radicale",)
         rs_servers = ()
