@@ -11,24 +11,28 @@ import pytest
 import vdirsyncer.exceptions as exceptions
 from vdirsyncer.storage.base import Item
 from vdirsyncer.storage.memory import MemoryStorage, _random_string
-from vdirsyncer.sync import BothReadOnly, IdentConflict, StorageEmpty, \
-    SyncConflict, sync
+from vdirsyncer.sync import BothReadOnly, IdentConflict, PartialSync, \
+    StorageEmpty, SyncConflict, sync
 
-from . import assert_item_equals, blow_up, uid_strategy
+from . import blow_up, uid_strategy
 
 
 def empty_storage(x):
     return list(x.list()) == []
 
 
+def items(s):
+    return set(x[1].raw for x in s.items.values())
+
+
 def test_irrelevant_status():
     a = MemoryStorage()
     b = MemoryStorage()
     status = {'1': ('1', 1234, '1.ics', 2345)}
     sync(a, b, status)
     assert not status
-    assert empty_storage(a)
-    assert empty_storage(b)
+    assert not items(a)
+    assert not items(b)
 
 
 def test_missing_status():
@@ -36,12 +40,11 @@ def test_missing_status():
     b = MemoryStorage()
     status = {}
     item = Item(u'asdf')
-    href_a, _ = a.upload(item)
-    href_b, _ = b.upload(item)
+    a.upload(item)
+    b.upload(item)
     sync(a, b, status)
     assert len(status) == 1
-    assert a.has(href_a)
-    assert b.has(href_b)
+    assert items(a) == items(b) == {item.raw}
 
 
 def test_missing_status_and_different_items():
@@ -57,8 +60,7 @@ def test_missing_status_and_different_items():
         sync(a, b, status)
     assert not status
     sync(a, b, status, conflict_resolution='a wins')
-    assert_item_equals(item1, b.get('1')[0])
-    assert_item_equals(item1, a.get('1')[0])
+    assert items(a) == items(b) == {item1.raw}
 
 
 def test_read_only_and_prefetch():
@@ -75,7 +77,52 @@ def test_read_only_and_prefetch():
     sync(a, b, status, force_delete=True)
     sync(a, b, status, force_delete=True)
 
-    assert list(a.list()) == list(b.list()) == []
+    assert not items(a) and not items(b)
+
+
+def test_partial_sync_ignore():
+    a = MemoryStorage()
+    b = MemoryStorage()
+    status = {}
+
+    item0 = Item('UID:0\nhehe')
+    a.upload(item0)
+    b.upload(item0)
+
+    b.read_only = True
+
+    item1 = Item('UID:1\nhaha')
+    a.upload(item1)
+
+    sync(a, b, status, partial_sync='ignore')
+    sync(a, b, status, partial_sync='ignore')
+
+    assert items(a) == {item0.raw, item1.raw}
+    assert items(b) == {item0.raw}
+
+
+def test_partial_sync_ignore2():
+    a = MemoryStorage()
+    b = MemoryStorage()
+    status = {}
+
+    href, etag = a.upload(Item('UID:0'))
+    a.read_only = True
+
+    sync(a, b, status, partial_sync='ignore', force_delete=True)
+    assert items(b) == items(a) == {'UID:0'}
+
+    b.items.clear()
+    sync(a, b, status, partial_sync='ignore', force_delete=True)
+    sync(a, b, status, partial_sync='ignore', force_delete=True)
+    assert items(a) == {'UID:0'}
+    assert not b.items
+
+    a.read_only = False
+    a.update(href, Item('UID:0\nupdated'), etag)
+    a.read_only = True
+    sync(a, b, status, partial_sync='ignore', force_delete=True)
+    assert items(b) == items(a) == {'UID:0\nupdated'}
 
 
 def test_upload_and_update():
@@ -86,22 +133,22 @@ def test_upload_and_update():
     item = Item(u'UID:1')  # new item 1 in a
     a.upload(item)
     sync(a, b, status)
-    assert_item_equals(b.get('1.b')[0], item)
+    assert items(b) == items(a) == {item.raw}
 
     item = Item(u'UID:1\nASDF:YES')  # update of item 1 in b
     b.update('1.b', item, b.get('1.b')[1])
     sync(a, b, status)
-    assert_item_equals(a.get('1.a')[0], item)
+    assert items(b) == items(a) == {item.raw}
 
     item2 = Item(u'UID:2')  # new item 2 in b
     b.upload(item2)
     sync(a, b, status)
-    assert_item_equals(a.get('2.a')[0], item2)
+    assert items(b) == items(a) == {item.raw, item2.raw}
 
     item2 = Item(u'UID:2\nASDF:YES')  # update of item 2 in a
     a.update('2.a', item2, a.get('2.a')[1])
     sync(a, b, status)
-    assert_item_equals(b.get('2.b')[0], item2)
+    assert items(b) == items(a) == {item.raw, item2.raw}
 
 
 def test_deletion():
@@ -111,18 +158,36 @@ def test_deletion():
 
     item = Item(u'UID:1')
     a.upload(item)
-    a.upload(Item(u'UID:2'))
+    item2 = Item(u'UID:2')
+    a.upload(item2)
     sync(a, b, status)
     b.delete('1.b', b.get('1.b')[1])
     sync(a, b, status)
-    assert not a.has('1.a') and not b.has('1.b')
+    assert items(a) == items(b) == {item2.raw}
 
     a.upload(item)
     sync(a, b, status)
-    assert a.has('1.a') and b.has('1.b')
+    assert items(a) == items(b) == {item.raw, item2.raw}
     a.delete('1.a', a.get('1.a')[1])
     sync(a, b, status)
-    assert not a.has('1.a') and not b.has('1.b')
+    assert items(a) == items(b) == {item2.raw}
+
+
+def test_insert_hash():
+    a = MemoryStorage()
+    b = MemoryStorage()
+    status = {}
+
+    item = Item('UID:1')
+    href, etag = a.upload(item)
+    sync(a, b, status)
+
+    for d in status['1']:
+        del d['hash']
+
+    a.update(href, Item('UID:1\nHAHA:YES'), etag)
+    sync(a, b, status)
+    assert 'hash' in status['1'][0] and 'hash' in status['1'][1]
 
 
 def test_already_synced():
@@ -134,20 +199,22 @@ def test_already_synced():
     status = {
         '1': ({
             'href': '1.a',
+            'hash': item.hash,
             'etag': a.get('1.a')[1]
         }, {
             'href': '1.b',
+            'hash': item.hash,
             'etag': b.get('1.b')[1]
         })
     }
-    old_status = dict(status)
+    old_status = deepcopy(status)
     a.update = b.update = a.upload = b.upload = \
         lambda *a, **kw: pytest.fail('Method shouldn\'t have been called.')
 
     for i in (1, 2):
         sync(a, b, status)
         assert status == old_status
-        assert a.has('1.a') and b.has('1.b')
+        assert items(a) == items(b) == {item.raw}
 
 
 @pytest.mark.parametrize('winning_storage', 'ab')
@@ -160,17 +227,16 @@ def test_conflict_resolution_both_etags_new(winning_storage):
     status = {}
     sync(a, b, status)
     assert status
-    a.update(href_a, Item(u'UID:1\nitem a'), etag_a)
-    b.update(href_b, Item(u'UID:1\nitem b'), etag_b)
+    item_a = Item(u'UID:1\nitem a')
+    item_b = Item(u'UID:1\nitem b')
+    a.update(href_a, item_a, etag_a)
+    b.update(href_b, item_b, etag_b)
     with pytest.raises(SyncConflict):
         sync(a, b, status)
     sync(a, b, status, conflict_resolution='{} wins'.format(winning_storage))
-    item_a, _ = a.get(href_a)
-    item_b, _ = b.get(href_b)
-    assert_item_equals(item_a, item_b)
-    n = item_a.raw.splitlines()
-    assert u'UID:1' in n
-    assert u'item {}'.format(winning_storage) in n
+    assert items(a) == items(b) == {
+        item_a.raw if winning_storage == 'a' else item_b.raw
+    }
 
 
 def test_updated_and_deleted():
@@ -182,10 +248,11 @@ def test_updated_and_deleted():
 
     (href_b, etag_b), = b.list()
     b.delete(href_b, etag_b)
-    a.update(href_a, Item(u'UID:1\nupdated'), etag_a)
+    updated = Item(u'UID:1\nupdated')
+    a.update(href_a, updated, etag_a)
     sync(a, b, status, force_delete=True)
 
-    assert len(list(a.list())) == len(list(b.list())) == 1
+    assert items(a) == items(b) == {updated.raw}
 
 
 def test_conflict_resolution_invalid_mode():
@@ -261,15 +328,11 @@ def test_empty_storage_dataloss():
 def test_no_uids():
     a = MemoryStorage()
     b = MemoryStorage()
-    href_a, _ = a.upload(Item(u'ASDF'))
-    href_b, _ = b.upload(Item(u'FOOBAR'))
+    a.upload(Item(u'ASDF'))
+    b.upload(Item(u'FOOBAR'))
     status = {}
     sync(a, b, status)
-
-    a_items = set(a.get(href)[0].raw for href, etag in a.list())
-    b_items = set(b.get(href)[0].raw for href, etag in b.list())
-
-    assert a_items == b_items == {u'ASDF', u'FOOBAR'}
+    assert items(a) == items(b) == {u'ASDF', u'FOOBAR'}
 
 
 def test_changed_uids():
@@ -304,9 +367,9 @@ def test_readonly():
     with pytest.raises(exceptions.ReadOnlyError):
         b.upload(Item(u'UID:3'))
 
-    sync(a, b, status)
+    sync(a, b, status, partial_sync='revert')
     assert len(status) == 2 and a.has(href_a) and not b.has(href_a)
-    sync(a, b, status)
+    sync(a, b, status, partial_sync='revert')
     assert len(status) == 1 and not a.has(href_a) and not b.has(href_a)
 
 
@@ -351,7 +414,7 @@ def test_moved_href():
 
     sync(a, b, status)
     assert len(status) == 1
-    assert len(list(a.list())) == len(list(b.list())) == 1
+    assert items(a) == items(b) == {'UID:haha'}
     assert status['haha'][1]['href'] == 'lol'
     old_status = deepcopy(status)
 
@@ -360,7 +423,7 @@ def test_moved_href():
 
     sync(a, b, status)
     assert old_status == status
-    assert len(list(a.list())) == len(list(b.list())) == 1
+    assert items(a) == items(b) == {'UID:haha'}
 
 
 def test_bogus_etag_change():
@@ -384,8 +447,8 @@ def test_bogus_etag_change():
     b.delete = b.update = b.upload = blow_up
 
     sync(a, b, status)
-    assert len(status) == len(list(a.list())) == len(list(b.list())) == 1
-    assert a.get(href_a)[0].raw == u'UID:ASDASD\nACTUALCHANGE:YES'
+    assert len(status) == 1
+    assert items(a) == items(b) == {u'UID:ASDASD\nACTUALCHANGE:YES'}
 
 
 def test_unicode_hrefs():
@@ -396,14 +459,18 @@ def test_unicode_hrefs():
     sync(a, b, status)
 
 
+class ActionIntentionallyFailed(Exception):
+    pass
+
+
+def action_failure(*a, **kw):
+    raise ActionIntentionallyFailed()
+
+
 class SyncMachine(RuleBasedStateMachine):
     Status = Bundle('status')
     Storage = Bundle('storage')
 
-    @staticmethod
-    def _get_items(storage):
-        return sorted(item.raw for etag, item in storage.items.values())
-
     @rule(target=Storage,
           read_only=st.booleans(),
           flaky_etags=st.booleans(),
@@ -413,7 +480,7 @@ class SyncMachine(RuleBasedStateMachine):
         s.read_only = read_only
         if flaky_etags:
             def get(href):
-                _, item = s.items[href]
+                old_etag, item = s.items[href]
                 etag = _random_string()
                 s.items[href] = etag, item
                 return item, etag
@@ -422,37 +489,57 @@ class SyncMachine(RuleBasedStateMachine):
         if null_etag_on_upload:
             _old_upload = s.upload
             _old_update = s.update
-            s.upload = lambda item: (_old_upload(item)[0], None)
-            s.update = lambda h, i, e: _old_update(h, i, e) and None
+            s.upload = lambda item: (_old_upload(item)[0], 'NULL')
+            s.update = lambda h, i, e: _old_update(h, i, e) and 'NULL'
 
         return s
 
+    @rule(s=Storage)
+    def actions_fail(self, s):
+        s.upload = action_failure
+        s.update = action_failure
+        s.delete = action_failure
+
     @rule(target=Status)
     def newstatus(self):
         return {}
 
-    @rule(target=Storage, storage=Storage,
+    @rule(storage=Storage,
           uid=uid_strategy,
           etag=st.text())
     def upload(self, storage, uid, etag):
         item = Item(u'UID:{}'.format(uid))
         storage.items[uid] = (etag, item)
-        return storage
 
-    @rule(target=Storage, storage=Storage, href=st.text())
+    @rule(storage=Storage, href=st.text())
     def delete(self, storage, href):
-        storage.items.pop(href, None)
-        return storage
+        assume(storage.items.pop(href, None))
 
     @rule(
-        target=Status, status=Status,
+        status=Status,
         a=Storage, b=Storage,
         force_delete=st.booleans(),
-        conflict_resolution=st.one_of((st.just('a wins'), st.just('b wins')))
+        conflict_resolution=st.one_of((st.just('a wins'), st.just('b wins'))),
+        with_error_callback=st.booleans(),
+        partial_sync=st.one_of((
+            st.just('ignore'), st.just('revert'), st.just('error')
+        ))
     )
-    def sync(self, status, a, b, force_delete, conflict_resolution):
-        old_items_a = self._get_items(a)
-        old_items_b = self._get_items(b)
+    def sync(self, status, a, b, force_delete, conflict_resolution,
+             with_error_callback, partial_sync):
+        assume(a is not b)
+        old_items_a = items(a)
+        old_items_b = items(b)
+
+        a.instance_name = 'a'
+        b.instance_name = 'b'
+
+        errors = []
+
+        if with_error_callback:
+            error_callback = errors.append
+        else:
+            error_callback = None
 
         try:
             # If one storage is read-only, double-sync because changes don't
@@ -460,7 +547,16 @@ class SyncMachine(RuleBasedStateMachine):
             for _ in range(2 if a.read_only or b.read_only else 1):
                 sync(a, b, status,
                      force_delete=force_delete,
-                     conflict_resolution=conflict_resolution)
+                     conflict_resolution=conflict_resolution,
+                     error_callback=error_callback,
+                     partial_sync=partial_sync)
+
+            for e in errors:
+                raise e
+        except PartialSync:
+            assert partial_sync == 'error'
+        except ActionIntentionallyFailed:
+            pass
         except BothReadOnly:
             assert a.read_only and b.read_only
             assume(False)
@@ -469,16 +565,43 @@ class SyncMachine(RuleBasedStateMachine):
                 raise
             else:
                 assert not list(a.list()) or not list(b.list())
-                return status
+        else:
+            items_a = items(a)
+            items_b = items(b)
 
-        items_a = self._get_items(a)
-        items_b = self._get_items(b)
+            assert items_a == items_b or partial_sync == 'ignore'
+            assert items_a == old_items_a or not a.read_only
+            assert items_b == old_items_b or not b.read_only
 
-        assert items_a == items_b
-        assert items_a == old_items_a or not a.read_only
-        assert items_b == old_items_b or not b.read_only
-
-        return status
+            assert set(a.items) | set(b.items) == set(status) or \
+                partial_sync == 'ignore'
 
 
 TestSyncMachine = SyncMachine.TestCase
+
+
+@pytest.mark.parametrize('error_callback', [True, False])
+def test_rollback(error_callback):
+    a = MemoryStorage()
+    b = MemoryStorage()
+    status = {}
+
+    a.items['0'] = ('', Item('UID:0'))
+    b.items['1'] = ('', Item('UID:1'))
+
+    b.upload = b.update = b.delete = action_failure
+
+    if error_callback:
+        errors = []
+
+        sync(a, b, status=status, conflict_resolution='a wins',
+             error_callback=errors.append)
+
+        assert len(errors) == 1
+        assert isinstance(errors[0], ActionIntentionallyFailed)
+
+        assert len(status) == 1
+        assert status['1']
+    else:
+        with pytest.raises(ActionIntentionallyFailed):
+            sync(a, b, status=status, conflict_resolution='a wins')
