@@ -8,8 +8,7 @@ import click
 
 import click_log
 
-from .. import PROJECT_HOME, __version__, exceptions
-from ..utils.compat import PY2
+from .. import __version__
 
 
 cli_logger = logging.getLogger(__name__)
@@ -30,7 +29,7 @@ def catch_errors(f):
     def inner(*a, **kw):
         try:
             f(*a, **kw)
-        except:
+        except BaseException:
             from .utils import handle_cli_error
             handle_cli_error()
             sys.exit(1)
@@ -38,29 +37,6 @@ def catch_errors(f):
     return inner
 
 
-def _check_python2(config):
-    # XXX: Py2
-    if not PY2:
-        return
-
-    msg = (
-        'Python 2 support will be dropped. Please switch '
-        'to at least Python 3.3 as soon as possible. See '
-        '{home}/issues/219 for more information.'
-        .format(home=PROJECT_HOME)
-    )
-
-    if not config.general.get('python2', False):
-        raise exceptions.UserError(
-            msg + (
-                '\nSet python2 = true in the [general] section to get rid of '
-                'this error for now.'
-            )
-        )
-    else:
-        cli_logger.warning(msg)
-
-
 @click.group()
 @click_log.init('vdirsyncer')
 @click_log.simple_verbosity_option()
@@ -72,11 +48,10 @@ def app(ctx, config):
     '''
     Synchronize calendars and contacts
     '''
-    from .config import load_config
 
     if not ctx.config:
+        from .config import load_config
         ctx.config = load_config(config)
-    _check_python2(ctx.config)
 
 main = app
 
@@ -252,16 +227,3 @@ def repair(ctx, collection):
                        'synchronization features.')
     click.confirm('Do you want to continue?', abort=True)
     repair_collection(ctx.config, collection)
-
-# Not sure if useful. I originally wanted it because:
-# * my password manager has a timeout for caching the master password
-# * when calling vdirsyncer in a cronjob, the master password prompt would
-#   randomly pop up
-# So I planned on piping a FIFO to vdirsyncer, and writing to that FIFO from a
-# cronjob.
-
-try:
-    import click_repl
-    click_repl.register_repl(app)
-except ImportError:
-    pass
