@@ -4,13 +4,12 @@ import functools
 import json
 
 from .config import CollectionConfig
-from .utils import JobFailed, cli_logger, coerce_native, \
-    collections_for_pair, get_status_name, handle_cli_error, load_status, \
-    save_status, storage_class_from_config, storage_instance_from_config
+from .utils import JobFailed, cli_logger, collections_for_pair, \
+    get_status_name, handle_cli_error, load_status, save_status, \
+    storage_class_from_config, storage_instance_from_config
 
 from .. import exceptions
 from ..sync import sync
-from ..utils.compat import to_unicode
 
 
 def prepare_pair(wq, pair_name, collections, config, callback, **kwargs):
@@ -23,9 +22,6 @@ def prepare_pair(wq, pair_name, collections, config, callback, **kwargs):
     # spawn one worker less because we can reuse the current one
     new_workers = -1
     for collection_name in (collections or all_collections):
-        # XXX: PY2 hack
-        if collection_name is not None:
-            collection_name = to_unicode(collection_name, 'utf-8')
         try:
             config_a, config_b = all_collections[collection_name]
         except KeyError:
@@ -51,27 +47,42 @@ def sync_collection(wq, collection, general, force_delete):
     status_name = get_status_name(pair.name, collection.name)
 
     try:
-        cli_logger.info('Syncing {}'.format(coerce_native(status_name)))
+        cli_logger.info('Syncing {}'.format(status_name))
 
         status = load_status(general['status_path'], pair.name,
                              collection.name, data_type='items') or {}
         cli_logger.debug('Loaded status for {}'
-                         .format(coerce_native(status_name)))
+                         .format(status_name))
 
         a = storage_instance_from_config(collection.config_a)
         b = storage_instance_from_config(collection.config_b)
+
+        sync_failed = False
+
+        def error_callback(e):
+            nonlocal sync_failed
+            sync_failed = True
+            handle_cli_error(status_name, e)
+
         sync(
             a, b, status,
-            conflict_resolution=pair.options.get('conflict_resolution', None),
-            force_delete=force_delete
+            conflict_resolution=pair.conflict_resolution,
+            force_delete=force_delete,
+            error_callback=error_callback,
+            partial_sync=pair.partial_sync
         )
-    except:
+
+        save_status(general['status_path'], pair.name, collection.name,
+                    data_type='items', data=status)
+
+        if sync_failed:
+            raise JobFailed()
+    except JobFailed:
+        raise
+    except BaseException:
         handle_cli_error(status_name)
         raise JobFailed()
 
-    save_status(general['status_path'], pair.name, collection.name,
-                data_type='items', data=status)
-
 
 def discover_collections(wq, pair, **kwargs):
     rv = collections_for_pair(pair=pair, **kwargs)
@@ -128,10 +139,10 @@ def metasync_collection(wq, collection, general):
 
         metasync(
             a, b, status,
-            conflict_resolution=pair.options.get('conflict_resolution', None),
-            keys=pair.options.get('metadata', None) or ()
+            conflict_resolution=pair.conflict_resolution,
+            keys=pair.metadata
         )
-    except:
+    except BaseException:
         handle_cli_error(status_name)
         raise JobFailed()
 
