@@ -2,9 +2,11 @@
 
 import datetime
 import logging
-
+import urllib.parse as urlparse
 import xml.etree.ElementTree as etree
 
+from inspect import getfullargspec
+
 import requests
 from requests.exceptions import HTTPError
 
@@ -12,15 +14,21 @@ from .base import Item, Storage, normalize_meta_value
 from .http import HTTP_STORAGE_PARAMETERS, USERAGENT, prepare_auth, \
     prepare_client_cert, prepare_verify
 from .. import exceptions, utils
-from ..utils.compat import PY2, getargspec_ish, text_type, to_native
 
 
 dav_logger = logging.getLogger(__name__)
 
 CALDAV_DT_FORMAT = '%Y%m%dT%H%M%SZ'
 
-_path_reserved_chars = frozenset(utils.compat.urlquote(x, '')
-                                 for x in "/?#[]!$&'()*+,;=")
+
+def _generate_path_reserved_chars():
+    for x in "/?#[]!$&'()*+,;":
+        x = urlparse.quote(x, '')
+        yield x.upper()
+        yield x.lower()
+
+_path_reserved_chars = frozenset(_generate_path_reserved_chars())
+del _generate_path_reserved_chars
 
 
 def _contains_quoted_reserved_chars(x):
@@ -35,13 +43,11 @@ def _normalize_href(base, href):
     '''Normalize the href to be a path only relative to hostname and
     schema.'''
     orig_href = href
-    base = to_native(base, 'utf-8')
-    href = to_native(href, 'utf-8')
     if not href:
         raise ValueError(href)
 
-    x = utils.compat.urlparse.urljoin(base, href)
-    x = utils.compat.urlparse.urlsplit(x).path
+    x = urlparse.urljoin(base, href)
+    x = urlparse.urlsplit(x).path
 
     # Encoding issues:
     # - https://github.com/owncloud/contacts/issues/581
@@ -51,9 +57,9 @@ def _normalize_href(base, href):
         if _contains_quoted_reserved_chars(x):
             break
         old_x = x
-        x = utils.compat.urlunquote(x)
+        x = urlparse.unquote(x)
 
-    x = utils.compat.urlquote(x, '/@%:')
+    x = urlparse.quote(x, '/@%:')
 
     if orig_href == x:
         dav_logger.debug('Already normalized: {!r}'.format(x))
@@ -104,7 +110,7 @@ class Discover(object):
     _homeset_xml = None
     _homeset_tag = None
     _well_known_uri = None
-    _collection_xml = """
+    _collection_xml = b"""
     <d:propfind xmlns:d="DAV:">
         <d:prop>
             <d:resourcetype />
@@ -122,7 +128,7 @@ class Discover(object):
     @staticmethod
     def _get_collection_from_url(url):
         _, collection = url.rstrip('/').rsplit('/', 1)
-        return utils.compat.urlunquote(collection)
+        return urlparse.unquote(collection)
 
     def find_dav(self):
         try:
@@ -139,14 +145,11 @@ class Discover(object):
 
     def find_principal(self, url=None):
         if url is None:
-            try:
-                return self.find_principal('')
-            except (HTTPError, exceptions.Error):
-                return self.find_principal(self.find_dav())
+            url = self.find_dav()
 
         headers = self.session.get_default_headers()
         headers['Depth'] = '0'
-        body = """
+        body = b"""
         <d:propfind xmlns:d="DAV:">
             <d:prop>
                 <d:current-user-principal />
@@ -158,8 +161,14 @@ class Discover(object):
         root = _parse_xml(response.content)
         rv = root.find('.//{DAV:}current-user-principal/{DAV:}href')
         if rv is None:
-            raise InvalidXMLResponse()
-        return utils.compat.urlparse.urljoin(response.url, rv.text)
+            # This is for servers that don't support current-user-principal
+            # E.g. Synology NAS
+            # See https://github.com/pimutils/vdirsyncer/issues/498
+            dav_logger.debug(
+                'No current-user-principal returned, re-using URL {}'
+                .format(response.url))
+            return response.url
+        return urlparse.urljoin(response.url, rv.text)
 
     def find_home(self, url=None):
         if url is None:
@@ -175,11 +184,21 @@ class Discover(object):
         rv = root.find('.//' + self._homeset_tag + '/{DAV:}href')
         if rv is None:
             raise InvalidXMLResponse('Couldn\'t find home-set.')
-        return utils.compat.urlparse.urljoin(response.url, rv.text)
+        return urlparse.urljoin(response.url, rv.text)
 
-    def find_collections(self, url=None):
-        if url is None:
-            url = self.find_home()
+    def find_collections(self):
+        rv = None
+        try:
+            rv = list(self._find_collections_impl(''))
+        except (HTTPError, exceptions.Error):
+            pass
+
+        if rv:
+            return rv
+        dav_logger.debug('Given URL is not a homeset URL')
+        return self._find_collections_impl(self.find_home())
+
+    def _find_collections_impl(self, url):
         headers = self.session.get_default_headers()
         headers['Depth'] = '1'
         r = self.session.request('PROPFIND', url, headers=headers,
@@ -195,7 +214,7 @@ class Discover(object):
             if href is None:
                 raise InvalidXMLResponse('Missing href tag for collection '
                                          'props.')
-            href = utils.compat.urlparse.urljoin(r.url, href.text)
+            href = urlparse.urljoin(r.url, href.text)
             if href not in done:
                 done.add(href)
                 yield {'href': href}
@@ -217,9 +236,9 @@ class Discover(object):
                 return c
 
         home = self.find_home()
-        url = utils.compat.urlparse.urljoin(
+        url = urlparse.urljoin(
             home,
-            utils.compat.urlquote(collection, '/@')
+            urlparse.quote(collection, '/@')
         )
 
         try:
@@ -245,8 +264,9 @@ class Discover(object):
                 </D:set>
             </D:mkcol>
         '''.format(
-            to_native(etree.tostring(etree.Element(self._resourcetype)))
-        )
+            etree.tostring(etree.Element(self._resourcetype),
+                           encoding='unicode')
+        ).encode('utf-8')
 
         response = self.session.request(
             'MKCOL',
@@ -260,7 +280,7 @@ class Discover(object):
 class CalDiscover(Discover):
     _namespace = 'urn:ietf:params:xml:ns:caldav'
     _resourcetype = '{%s}calendar' % _namespace
-    _homeset_xml = """
+    _homeset_xml = b"""
     <d:propfind xmlns:d="DAV:" xmlns:c="urn:ietf:params:xml:ns:caldav">
         <d:prop>
             <c:calendar-home-set />
@@ -268,13 +288,13 @@ class CalDiscover(Discover):
     </d:propfind>
     """
     _homeset_tag = '{%s}calendar-home-set' % _namespace
-    _well_known_uri = '/.well-known/caldav/'
+    _well_known_uri = '/.well-known/caldav'
 
 
 class CardDiscover(Discover):
     _namespace = 'urn:ietf:params:xml:ns:carddav'
     _resourcetype = '{%s}addressbook' % _namespace
-    _homeset_xml = """
+    _homeset_xml = b"""
     <d:propfind xmlns:d="DAV:" xmlns:c="urn:ietf:params:xml:ns:carddav">
         <d:prop>
             <c:addressbook-home-set />
@@ -282,17 +302,17 @@ class CardDiscover(Discover):
     </d:propfind>
     """
     _homeset_tag = '{%s}addressbook-home-set' % _namespace
-    _well_known_uri = '/.well-known/carddav/'
+    _well_known_uri = '/.well-known/carddav'
 
 
-class DavSession(object):
+class DAVSession(object):
     '''
     A helper class to connect to DAV servers.
     '''
 
     @classmethod
     def init_and_remaining_args(cls, **kwargs):
-        argspec = getargspec_ish(cls.__init__)
+        argspec = getfullargspec(cls.__init__)
         self_args, remainder = \
             utils.split_dict(kwargs, argspec.args.__contains__)
 
@@ -314,12 +334,12 @@ class DavSession(object):
 
     @utils.cached_property
     def parsed_url(self):
-        return utils.compat.urlparse.urlparse(self.url)
+        return urlparse.urlparse(self.url)
 
     def request(self, method, path, **kwargs):
         url = self.url
         if path:
-            url = utils.compat.urlparse.urljoin(self.url, path)
+            url = urlparse.urljoin(self.url, path)
 
         more = dict(self._settings)
         more.update(kwargs)
@@ -332,7 +352,7 @@ class DavSession(object):
         }
 
 
-class DavStorage(Storage):
+class DAVStorage(Storage):
 
     __doc__ = '''
     :param url: Base URL or an URL to a collection.
@@ -354,8 +374,8 @@ class DavStorage(Storage):
     get_multi_data_query = None
     # The Discover subclass to use
     discovery_class = None
-    # The DavSession class to use
-    session_class = DavSession
+    # The DAVSession class to use
+    session_class = DAVSession
 
     _repr_attributes = ('username', 'url')
 
@@ -370,11 +390,10 @@ class DavStorage(Storage):
 
         self.session, kwargs = \
             self.session_class.init_and_remaining_args(**kwargs)
-        super(DavStorage, self).__init__(**kwargs)
+        super(DAVStorage, self).__init__(**kwargs)
 
-    if not PY2:
-        import inspect
-        __init__.__signature__ = inspect.signature(session_class.__init__)
+    import inspect
+    __init__.__signature__ = inspect.signature(session_class.__init__)
 
     @classmethod
     def discover(cls, **kwargs):
@@ -413,7 +432,8 @@ class DavStorage(Storage):
         if not href_xml:
             return ()
 
-        data = self.get_multi_template.format(hrefs='\n'.join(href_xml))
+        data = self.get_multi_template \
+            .format(hrefs='\n'.join(href_xml)).encode('utf-8')
         response = self.session.request(
             'REPORT',
             '',
@@ -479,10 +499,9 @@ class DavStorage(Storage):
         #
         # -- https://tools.ietf.org/html/rfc7231#section-4.3.4
         #
-        # In such cases we return None as etag. The next synchronization will
-        # then detect an etag change (None != some string) and will download
-        # the new item.
-        etag = response.headers.get('etag', None)
+        # In such cases we return a constant etag. The next synchronization
+        # will then detect an etag change and will download the new item.
+        etag = response.headers.get('etag', 'NULL')
         href = self._normalize_href(response.url)
         return href, etag
 
@@ -570,7 +589,7 @@ class DavStorage(Storage):
                     <D:getetag/>
                 </D:prop>
             </D:propfind>
-            '''
+            '''.encode('utf-8')
 
         # We use a PROPFIND request instead of addressbook-query due to issues
         # with Zimbra. See https://github.com/pimutils/vdirsyncer/issues/83
@@ -596,8 +615,8 @@ class DavStorage(Storage):
                 </D:prop>
             </D:propfind>
         '''.format(
-            to_native(etree.tostring(etree.Element(xpath)))
-        )
+            etree.tostring(etree.Element(xpath), encoding='unicode')
+        ).encode('utf-8')
 
         headers = self.session.get_default_headers()
         headers['Depth'] = '0'
@@ -633,7 +652,7 @@ class DavStorage(Storage):
                     </D:prop>
                 </D:set>
             </D:propertyupdate>
-        '''.format(to_native(etree.tostring(element)))
+        '''.format(etree.tostring(element, encoding='unicode')).encode('utf-8')
 
         self.session.request(
             'PROPPATCH', '',
@@ -647,7 +666,7 @@ class DavStorage(Storage):
         # a PROPFIND to see if the value got actually set.
 
 
-class CaldavStorage(DavStorage):
+class CalDAVStorage(DAVStorage):
 
     __doc__ = '''
     CalDAV.
@@ -676,7 +695,7 @@ class CaldavStorage(DavStorage):
     :param item_types: Kind of items to show. The default, the empty list, is
         to show all. This depends on particular features on the server, the
         results are not validated.
-    ''' + DavStorage.__doc__
+    ''' + DAVStorage.__doc__
 
     storage_name = 'caldav'
     fileext = '.ics'
@@ -698,14 +717,14 @@ class CaldavStorage(DavStorage):
 
     get_multi_data_query = '{urn:ietf:params:xml:ns:caldav}calendar-data'
 
-    _property_table = dict(DavStorage._property_table)
+    _property_table = dict(DAVStorage._property_table)
     _property_table.update({
         'color': ('calendar-color', 'http://apple.com/ns/ical/'),
     })
 
     def __init__(self, start_date=None, end_date=None,
                  item_types=(), **kwargs):
-        super(CaldavStorage, self).__init__(**kwargs)
+        super(CalDAVStorage, self).__init__(**kwargs)
         if not isinstance(item_types, (list, tuple)):
             raise exceptions.UserError('item_types must be a list.')
 
@@ -717,11 +736,11 @@ class CaldavStorage(DavStorage):
             namespace = dict(datetime.__dict__)
             namespace['start_date'] = self.start_date = \
                 (eval(start_date, namespace)
-                 if isinstance(start_date, (bytes, text_type))
+                 if isinstance(start_date, (bytes, str))
                  else start_date)
             self.end_date = \
                 (eval(end_date, namespace)
-                 if isinstance(end_date, (bytes, text_type))
+                 if isinstance(end_date, (bytes, str))
                  else end_date)
 
     @staticmethod
@@ -749,7 +768,7 @@ class CaldavStorage(DavStorage):
                                           timefilter=timefilter)
         else:
             if start is not None and end is not None:
-                for x in CaldavStorage._get_list_filters(('VTODO', 'VEVENT'),
+                for x in CalDAVStorage._get_list_filters(('VTODO', 'VEVENT'),
                                                          start, end):
                     yield x
 
@@ -767,7 +786,7 @@ class CaldavStorage(DavStorage):
             # instead?
             #
             # See https://github.com/dmfs/tasks/issues/118 for backstory.
-            for x in DavStorage.list(self):
+            for x in DAVStorage.list(self):
                 yield x
 
         data = '''<?xml version="1.0" encoding="utf-8" ?>
@@ -790,7 +809,7 @@ class CaldavStorage(DavStorage):
         headers['Depth'] = '1'
 
         for caldavfilter in caldavfilters:
-            xml = data.format(caldavfilter=caldavfilter)
+            xml = data.format(caldavfilter=caldavfilter).encode('utf-8')
             response = self.session.request('REPORT', '', data=xml,
                                             headers=headers)
             root = _parse_xml(response.content)
@@ -799,11 +818,11 @@ class CaldavStorage(DavStorage):
                 yield href, etag
 
 
-class CarddavStorage(DavStorage):
+class CardDAVStorage(DAVStorage):
 
     __doc__ = '''
     CardDAV.
-    ''' + DavStorage.__doc__
+    ''' + DAVStorage.__doc__
 
     storage_name = 'carddav'
     fileext = '.vcf'
