@@ -1,9 +1,10 @@
 # -*- coding: utf-8 -*-
 
+import urllib.parse as urlparse
+
 from .base import Item, Storage
 from .. import exceptions
 from ..utils import expand_path
-from ..utils.compat import iteritems, text_type, urlparse
 from ..utils.http import request
 from ..utils.vobject import split_collection
 
@@ -39,7 +40,7 @@ def prepare_auth(auth, username, password):
 
 
 def prepare_verify(verify, verify_fingerprint):
-    if isinstance(verify, (text_type, bytes)):
+    if isinstance(verify, (str, bytes)):
         verify = expand_path(verify)
     elif not isinstance(verify, bool):
         raise exceptions.UserError('Invalid value for verify ({}), '
@@ -47,7 +48,7 @@ def prepare_verify(verify, verify_fingerprint):
                                    .format(verify))
 
     if verify_fingerprint is not None:
-        if not isinstance(verify_fingerprint, (bytes, text_type)):
+        if not isinstance(verify_fingerprint, (bytes, str)):
             raise exceptions.UserError('Invalid value for verify_fingerprint '
                                        '({}), must be a string or null.'
                                        .format(verify_fingerprint))
@@ -64,7 +65,7 @@ def prepare_verify(verify, verify_fingerprint):
 
 
 def prepare_client_cert(cert):
-    if isinstance(cert, (text_type, bytes)):
+    if isinstance(cert, (str, bytes)):
         cert = expand_path(cert)
     elif isinstance(cert, list):
         cert = tuple(map(prepare_client_cert, cert))
@@ -94,6 +95,16 @@ HTTP_STORAGE_PARAMETERS = '''
 class HttpStorage(Storage):
     __doc__ = '''
     Use a simple ``.ics`` file (or similar) from the web.
+    ``webcal://``-calendars are supposed to be used with this, but you have to
+    replace ``webcal://`` with ``http://``, or better, ``https://``.
+
+    Too many WebCAL providers generate UIDs of all ``VEVENT``-components
+    on-the-fly, i.e. all UIDs change every time the calendar is downloaded.
+    This leads many synchronization programs to believe that all events have
+    been deleted and new ones created, and accordingly causes a lot of
+    unnecessary uploads and deletions on the other side. Vdirsyncer completely
+    ignores UIDs coming from :storage:`http` and will replace them with a hash
+    of the normalized item content.
 
     :param url: URL to the ``.ics`` file.
     ''' + HTTP_STORAGE_PARAMETERS + '''
@@ -103,14 +114,15 @@ class HttpStorage(Storage):
         [pair holidays]
         a = holidays_local
         b = holidays_remote
+        collections = null
 
         [storage holidays_local]
-        type = filesystem
+        type = "filesystem"
         path = ~/.config/vdir/calendars/holidays/
         fileext = .ics
 
         [storage holidays_remote]
-        type = http
+        type = "http"
         url = https://example.com/holidays_from_hicksville.ics
     '''
 
@@ -119,6 +131,9 @@ class HttpStorage(Storage):
     _repr_attributes = ('username', 'url')
     _items = None
 
+    # Required for tests.
+    _ignore_uids = True
+
     def __init__(self, url, username='', password='', verify=True, auth=None,
                  useragent=USERAGENT, verify_fingerprint=None, auth_cert=None,
                  **kwargs):
@@ -150,10 +165,12 @@ class HttpStorage(Storage):
 
         for item in split_collection(r.text):
             item = Item(item)
-            etag = item.hash
-            self._items[item.ident] = item, etag
+            if self._ignore_uids:
+                item = item.with_uid(item.hash)
 
-        return ((href, etag) for href, (item, etag) in iteritems(self._items))
+            self._items[item.ident] = item, item.hash
+
+        return ((href, etag) for href, (item, etag) in self._items.items())
 
     def get(self, href):
         if self._items is None:
