@@ -4,18 +4,9 @@ import hashlib
 from itertools import chain, tee
 
 from . import cached_property, uniq
-from .compat import itervalues, text_type
 
 
-def _process_properties(*s):
-    rv = set()
-    for key in s:
-        rv.add(key + ':')
-        rv.add(key + ';')
-
-    return tuple(rv)
-
-IGNORE_PROPS = _process_properties(
+IGNORE_PROPS = (
     # PRODID is changed by radicale for some reason after upload
     'PRODID',
     # X-RADICALE-NAME is used by radicale, because hrefs don't really exist in
@@ -24,21 +15,20 @@ IGNORE_PROPS = _process_properties(
     # Apparently this is set by Horde?
     # https://github.com/pimutils/vdirsyncer/issues/318
     'X-WR-CALNAME',
-    # REV is from the VCARD specification and is supposed to change when the
+    # Those are from the VCARD specification and is supposed to change when the
     # item does -- however, we can determine that ourselves
-    # Same with LAST-MODIFIED
     'REV',
     'LAST-MODIFIED',
-    # Added those because e.g. http://www.feiertage-oesterreich.at/ is
-    # generating those randomly on every request.
-    # Some iCalendar HTTP calendars (Google's read-only calendar links)
-    # generate the DTSTAMP at request time, so this property always changes
-    # when the rest of the item didn't.
-    # Some do the same with the UID.
+    'CREATED',
+    # Some iCalendar HTTP calendars generate the DTSTAMP at request time, so
+    # this property always changes when the rest of the item didn't. Some do
+    # the same with the UID.
+    #
+    # - Google's read-only calendar links
+    # - http://www.feiertage-oesterreich.at/
     'DTSTAMP',
     'UID',
 )
-del _process_properties
 
 
 class Item(object):
@@ -47,9 +37,23 @@ class Item(object):
     VCARD'''
 
     def __init__(self, raw):
-        assert isinstance(raw, text_type)
+        assert isinstance(raw, str)
         self._raw = raw
 
+    def with_uid(self, new_uid):
+        parsed = _Component.parse(self.raw)
+        stack = [parsed]
+        while stack:
+            component = stack.pop()
+            stack.extend(component.subcomponents)
+
+            if component.name in ('VEVENT', 'VTODO', 'VJOURNAL', 'VCARD'):
+                del component['UID']
+                if new_uid:
+                    component['UID'] = new_uid
+
+        return Item('\r\n'.join(parsed.dump_lines()))
+
     @cached_property
     def raw(self):
         '''Raw content of the item, as unicode string.
@@ -89,8 +93,9 @@ class Item(object):
         # 2. The status file would contain really sensitive information.
         return self.uid or self.hash
 
-    @cached_property
+    @property
     def parsed(self):
+        '''Don't cache because the rv is mutable.'''
         try:
             return _Component.parse(self.raw)
         except Exception:
@@ -101,10 +106,13 @@ def normalize_item(item, ignore_props=IGNORE_PROPS):
     '''Create syntactically invalid mess that is equal for similar items.'''
     if not isinstance(item, Item):
         item = Item(item)
-    return u'\r\n'.join(line.strip()
-                        for line in sorted(item.raw.splitlines())
-                        if line.strip() and
-                        not line.startswith(IGNORE_PROPS))
+
+    x = _Component('TEMP', item.raw.splitlines(), [])
+    for prop in IGNORE_PROPS:
+        del x[prop]
+
+    x.props.sort()
+    return u'\r\n'.join(filter(bool, (line.strip() for line in x.props)))
 
 
 def hash_item(text):
@@ -112,7 +120,7 @@ def hash_item(text):
 
 
 def split_collection(text):
-    assert isinstance(text, text_type)
+    assert isinstance(text, str)
     inline = []
     items = {}  # uid => item
     ungrouped_items = []
@@ -142,7 +150,7 @@ def split_collection(text):
     for main in _Component.parse(text, multiple=True):
         inner(main, main)
 
-    for item in chain(itervalues(items), ungrouped_items):
+    for item in chain(items.values(), ungrouped_items):
         item.subcomponents.extend(inline)
         yield u'\r\n'.join(item.dump_lines())
 
@@ -241,7 +249,7 @@ class _Component(object):
     def parse(cls, lines, multiple=False):
         if isinstance(lines, bytes):
             lines = lines.decode('utf-8')
-        if isinstance(lines, text_type):
+        if isinstance(lines, str):
             lines = lines.splitlines()
 
         stack = []
@@ -282,36 +290,38 @@ class _Component(object):
         yield u'END:{}'.format(self.name)
 
     def __delitem__(self, key):
-        prefix = u'{}:'.format(key)
+        prefix = (u'{}:'.format(key), u'{};'.format(key))
         new_lines = []
         lineiter = iter(self.props)
-        for line in lineiter:
-            if line.startswith(prefix):
-                break
+        while True:
+            for line in lineiter:
+                if line.startswith(prefix):
+                    break
+                else:
+                    new_lines.append(line)
             else:
-                new_lines.append(line)
-
-        for line in lineiter:
-            if not line.startswith((u' ', u'\t')):
-                new_lines.append(line)
                 break
 
-        new_lines.extend(lineiter)
+            for line in lineiter:
+                if not line.startswith((u' ', u'\t')):
+                    new_lines.append(line)
+                    break
+
         self.props = new_lines
 
     def __setitem__(self, key, val):
-        assert isinstance(val, text_type)
+        assert isinstance(val, str)
         assert u'\n' not in val
         del self[key]
         line = u'{}:{}'.format(key, val)
         self.props.append(line)
 
     def __getitem__(self, key):
-        prefix = u'{}:'.format(key)
+        prefix = (u'{}:'.format(key), u'{};'.format(key))
         iterlines = iter(self.props)
         for line in iterlines:
             if line.startswith(prefix):
-                rv = line[len(prefix):]
+                rv = line.split(u':', 1)[-1]
                 break
         else:
             raise KeyError()
